#!/usr/local/bin/perl
# PDF-PL web demo
# Developed by Phil Smith
# Electronic Publishing Research Group, University of Nottingham
# http://www.ep.cs.nott.ac.uk/
# ep@cs.nott.ac.uk

# This script runs first time to create a form to display for the user
# to input number of sides of shape to display, and secondly is called
# from that form with parameters to create the PDF (see if condition below).

BEGIN {
  $CGI_LIB = '/var/apache/documents/pdf-pl/perl5-cgi/CGI';
  unshift (@INC, $CGI_LIB);
}


$tmpfilename = "/tmp/pdfpldemo$$.pdf";

$SIG{'PIPE'} = 'sigpipe';
$SIG{'INT'} = 'sigint';
sub sigpipe {
    unlink($tmpfilename);
    open(CAUGHT, ">/tmp/pdf-pl-$$-caught-SIGPIPE");
    close(CAUGHT);
    exit;
}
sub sigint {
    unlink($tmpfilename);
    open(CAUGHT, ">/tmp/pdf-pl-$$-caught-SIGINT");
    close(CAUGHT);
    exit;
}


use CGI;

# normal creation for when running as a CGI script
$query = new CGI;

# alternative creation for testing
#open(DUMMY, './dummydata');
#$query = new CGI(DUMMY);

# dump environment into /tmp/env (for debugging)
#open(DUMPENV, '>/tmp/env');
#foreach $key (sort keys %ENV) {
#    print DUMPENV "$key=$ENV{$key}\n";
#}
#close(DUMPENV);


# create form first time
# create PDF second time
if ($query->param) {
    process_form();
} else {
    create_form('PDF just for you');
}

sub pnserror {
    unlink($tmpfilename);
    create_form(@_);
    exit;
}

sub process_form {
    # respond to completed form by generating a PDF file

    use Pdf::File;
    use Pdf::Font;
    use Pdf::Graphic;


    my $pdf = Pdf::File->new();
    $pdf->pdfOpen ($tmpfilename);
    $pdf->setPageSize(400, 300);
    $pdf->newPage ();

    $g = $pdf->beginGraphic ();
    
    $n_sides = 0 + $query->param('sides');
    if ($n_sides < 2)	{
	pnserror('Try again!',
		 'Shapes usually have at least 2 sides!',
		 'Were you trying to confuse me?');
    }
    # note that you can have a non-integral number of sides.  Cool!

    $pi = 3.1415927;
    $ang = (2 * $pi) / $n_sides;
    $radius = 100;

    $g->gsave();
    $g->setColors('f', $query->param('color'));
    $g->ctm(0, 1, -1, 0, 200, 150);
    

    # draw the polygon path with sin and cos in a loop.  The path is closed
    # with closepath, but we still go right up to (floor of) $n_sides in case
    # we have fractionally more than a whole number of sides :-)

    $g->moveto($radius,0);
    for $i (1 .. $n_sides) {
	if ($i == $n_sides) { $i = 0; } # avoid rounding errors for final point
	$x = $radius * cos($i * $ang);
	$y = $radius * sin($i * $ang);
	$g->lineto($x, $y);
    }
    $g->closepath();
    $g->fill();
    
    $g->grestore();
    
    $greeting = "Hello, ".$query->param('name')."!";
    @textlines = ($greeting, "This is just for you!");
    $g->textBlockCenter(200, 160, 'Helvetica-Bold', 24, 30, @textlines);
    
    $progname = $ENV{'SCRIPT_NAME'};
    $server = $ENV{'SERVER_NAME'};

    if ($progname eq '') {$progname = 'a script';}
    if ($server eq '') {$server = '';} else {$server = 'http://'.$server;}

    $g->beginText('Helvetica', 6);
    $g->textRight(380, 20, "Generated by $server$progname at ".`/bin/date`);
    $g->endText();

    $g->endGraphic();
    
    $pdf->pdfClose();
    
    
    # return HTTP header
    print $query->header('application/pdf');
    #print "Content-type: application/pdf\n\n";
    
    # write PDF file
    open(PDFFILE, $tmpfilename);
    while(<PDFFILE>) {
	print $_;
    }
    close(PDFFILE);

    unlink($tmpfilename);

}


sub create_form {
    # generate empty form

    local ($title,@messages) = @_;

    $title = $title || 'Untitled';

    print $query->header;
    print $query->start_html($title);
    print "<H1>$title</H1>\n";
    print $query->startform;
    print "What's your name? ",$query->textfield('name','Matey');
    print "\n<P>What's your favourite colour? ",
    $query->popup_menu('color',['red','green','blue','yellow']);
    print "\n<P>And how many sides has your favourite shape? ",
    $query->textfield('sides','7',5,5);
    print "\n<P>",$query->submit('Make my day!');
    print $query->endform;

    print "<HR>\n" if @messages;
    foreach $mess (@messages) {
	print "<P>$mess\n";
    }

    print $query->end_html;
}
